@@ -2,11 +2,11 @@
  * -----------------------------------------------------------------
  * $Revision$
  * $Date$
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2020, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -19,7 +19,7 @@
  * -----------------------------------------------------------------
  */
 
-/* 
+/*
  * =================================================================
  * IMPORTED HEADER FILES
  * =================================================================
@@ -33,7 +33,7 @@
 #include <sundials/sundials_math.h>
 #include <sundials/sundials_types.h>
 
-/* 
+/*
  * =================================================================
  * CVODEA PRIVATE CONSTANTS
  * =================================================================
@@ -45,7 +45,7 @@
 #define HUNDRED     RCONST(100.0)      /* real 100.0 */
 #define FUZZ_FACTOR RCONST(1000000.0)  /* fuzz factor for IMget */
 
-/* 
+/*
  * =================================================================
  * PRIVATE FUNCTION PROTOTYPES
  * =================================================================
@@ -58,9 +58,9 @@ static void CVAckpntDelete(CkpntMem *ck_memPtr);
 static void CVAbckpbDelete(CVodeBMem *cvB_memPtr);
 
 static int  CVAdataStore(CVodeMem cv_mem, CkpntMem ck_mem);
-static int  CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem); 
+static int  CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem);
 
-static int CVAfindIndex(CVodeMem cv_mem, realtype t, 
+static int CVAfindIndex(CVodeMem cv_mem, realtype t,
                         long int *indx, booleantype *newpoint);
 
 static booleantype CVAhermiteMalloc(CVodeMem cv_mem);
@@ -75,13 +75,13 @@ static int CVApolynomialStorePnt(CVodeMem cv_mem, DtpntMem d);
 
 /* Wrappers */
 
-static int CVArhs(realtype t, N_Vector yB, 
+static int CVArhs(realtype t, N_Vector yB,
                   N_Vector yBdot, void *cvode_mem);
 
-static int CVArhsQ(realtype t, N_Vector yB, 
+static int CVArhsQ(realtype t, N_Vector yB,
                    N_Vector qBdot, void *cvode_mem);
 
-/* 
+/*
  * =================================================================
  * EXPORTED FUNCTIONS IMPLEMENTATION
  * =================================================================
@@ -90,7 +90,7 @@ static int CVArhsQ(realtype t, N_Vector yB,
 /*
  * CVodeAdjInit
  *
- * This routine initializes ASA and allocates space for the adjoint 
+ * This routine initializes ASA and allocates space for the adjoint
  * memory structure.
  */
 
@@ -118,7 +118,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
   if ( (interp != CV_HERMITE) && (interp != CV_POLYNOMIAL) ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeAdjInit", MSGCV_BAD_INTERP);
     return(CV_ILL_INPUT);
-  } 
+  }
 
   /* ----------------------------
    * Allocate CVODEA memory block
@@ -173,7 +173,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
     return(CV_MEM_FAIL);
   }
 
-  for (i=0; i<=steps; i++) { 
+  for (i=0; i<=steps; i++) {
     ca_mem->dt_mem[i] = NULL;
     ca_mem->dt_mem[i] = (DtpntMem) malloc(sizeof(struct DtpntMemRec));
     if (ca_mem->dt_mem[i] == NULL) {
@@ -186,20 +186,20 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
   }
 
   /* Attach functions for the appropriate interpolation module */
-  
+
   switch(interp) {
 
   case CV_HERMITE:
-    
+
     ca_mem->ca_IMmalloc = CVAhermiteMalloc;
     ca_mem->ca_IMfree   = CVAhermiteFree;
     ca_mem->ca_IMget    = CVAhermiteGetY;
     ca_mem->ca_IMstore  = CVAhermiteStorePnt;
 
     break;
-    
+
   case CV_POLYNOMIAL:
-  
+
     ca_mem->ca_IMmalloc = CVApolynomialMalloc;
     ca_mem->ca_IMfree   = CVApolynomialFree;
     ca_mem->ca_IMget    = CVApolynomialGetY;
@@ -215,7 +215,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
 
   /* By default we will store but not interpolate sensitivities
    *  - IMstoreSensi will be set in CVodeF to SUNFALSE if FSA is not enabled
-   *    or if the user can force this through CVodeSetAdjNoSensi 
+   *    or if the user can force this through CVodeSetAdjNoSensi
    *  - IMinterpSensi will be set in CVodeB to SUNTRUE if IMstoreSensi is
    *    SUNTRUE and if at least one backward problem requires sensitivities */
 
@@ -239,6 +239,8 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
 
   ca_mem->ca_firstCVodeBcall = SUNTRUE;
 
+  ca_mem->ca_rootret = SUNFALSE;
+
   /* ---------------------------------------------
    * ASA initialized and allocated
    * --------------------------------------------- */
@@ -247,7 +249,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
   cv_mem->cv_adjMallocDone = SUNTRUE;
 
   return(CV_SUCCESS);
-} 
+}
 
 /* CVodeAdjReInit
  *
@@ -255,7 +257,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
  * the number of steps between check points and the type of interpolation
  * remain unchanged.
  * The list of check points (and associated memory) is deleted.
- * The list of backward problems is kept (however, new backward problems can 
+ * The list of backward problems is kept (however, new backward problems can
  * be added to this list by calling CVodeCreateB).
  * The CVODES memory for the forward and backward problems can be reinitialized
  * separately by calling CVodeReInit and CVodeReInitB, respectively.
@@ -280,7 +282,7 @@ int CVodeAdjReInit(void *cvode_mem)
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeAdjReInit", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
 
   ca_mem = cv_mem->cv_adj_mem;
 
@@ -289,13 +291,13 @@ int CVodeAdjReInit(void *cvode_mem)
   while (ca_mem->ck_mem != NULL) CVAckpntDelete(&(ca_mem->ck_mem));
 
   /* Initialization of check points */
-  
+
   ca_mem->ck_mem = NULL;
   ca_mem->ca_nckpnts = 0;
   ca_mem->ca_ckpntData = NULL;
 
   /* CVodeF and CVodeB not called yet */
- 
+
   ca_mem->ca_firstCVodeFcall = SUNTRUE;
   ca_mem->ca_tstopCVodeFcall = SUNFALSE;
   ca_mem->ca_firstCVodeBcall = SUNTRUE;
@@ -314,7 +316,7 @@ void CVodeAdjFree(void *cvode_mem)
   CVodeMem cv_mem;
   CVadjMem ca_mem;
   long int i;
-  
+
   if (cvode_mem == NULL) return;
   cv_mem = (CVodeMem) cvode_mem;
 
@@ -351,14 +353,14 @@ void CVodeAdjFree(void *cvode_mem)
  * CVodeF
  *
  * This routine integrates to tout and returns solution into yout.
- * In the same time, it stores check point data every 'steps' steps. 
- * 
+ * In the same time, it stores check point data every 'steps' steps.
+ *
  * CVodeF can be called repeatedly by the user.
  *
  * ncheckPtr points to the number of check points stored so far.
  */
 
-int CVodeF(void *cvode_mem, realtype tout, N_Vector yout, 
+int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
            realtype *tret, int itask, int *ncheckPtr)
 {
   CVadjMem ca_mem;
@@ -366,7 +368,8 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
   CkpntMem tmp;
   DtpntMem *dt_mem;
   int flag, i;
-  booleantype iret, allocOK;
+  booleantype allocOK, earlyret;
+  realtype ttest;
 
   /* Check if cvode_mem exists */
   if (cvode_mem == NULL) {
@@ -379,7 +382,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeF", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
 
   ca_mem = cv_mem->cv_adj_mem;
 
@@ -388,7 +391,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeF", MSGCV_YOUT_NULL);
     return(CV_ILL_INPUT);
   }
-  
+
   /* Check for tret != NULL */
   if (tret == NULL) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeF", MSGCV_TRET_NULL);
@@ -411,17 +414,12 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
     ca_mem->ca_tstopCVodeF = cv_mem->cv_tstop;
   }
 
-  /* We will call CVode in CV_ONE_STEP mode, regardless
-   * of what itask is, so flag if we need to return */
-  if (itask == CV_ONE_STEP) iret = SUNTRUE;
-  else                      iret = SUNFALSE;
-
   /* On the first step:
    *   - set tinitial
    *   - initialize list of check points
    *   - if needed, initialize the interpolation module
    *   - load dt_mem[0]
-   * On subsequent steps, test if taking a new step is necessary. 
+   * On subsequent steps, test if taking a new step is necessary.
    */
   if ( ca_mem->ca_firstCVodeFcall ) {
 
@@ -460,18 +458,41 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
 
     ca_mem->ca_firstCVodeFcall = SUNFALSE;
 
-  } else if ( (cv_mem->cv_tn - tout)*cv_mem->cv_h >= ZERO ) {
+  } else if ( itask == CV_NORMAL ) {
 
-    /* If tout was passed, return interpolated solution. 
-       No changes to ck_mem or dt_mem are needed. */
-    *tret = tout;
-    flag = CVodeGetDky(cv_mem, tout, 0, yout);
-    *ncheckPtr = ca_mem->ca_nckpnts;
-    ca_mem->ca_IMnewData = SUNTRUE;
-    ca_mem->ca_ckpntData = ca_mem->ck_mem;
-    ca_mem->ca_np = cv_mem->cv_nst % ca_mem->ca_nsteps + 1;
+    /* When in normal mode, check if tout was passed or if a previous root was
+       not reported and return an interpolated solution. No changes to ck_mem
+       or dt_mem are needed. */
 
-    return(flag);
+    /* flag to signal if an early return is needed */
+    earlyret = SUNFALSE;
+
+    /* if a root needs to be reported compare tout to troot otherwise compare
+       to the current time tn */
+    ttest = (ca_mem->ca_rootret) ? ca_mem->ca_troot : cv_mem->cv_tn;
+
+    if ((ttest - tout)*cv_mem->cv_h >= ZERO) {
+      /* ttest is after tout, interpolate to tout */
+      *tret = tout;
+      flag = CVodeGetDky(cv_mem, tout, 0, yout);
+      earlyret = SUNTRUE;
+    } else if (ca_mem->ca_rootret) {
+      /* tout is after troot, interpolate to troot */
+      *tret = ca_mem->ca_troot;
+      flag = CVodeGetDky(cv_mem, ca_mem->ca_troot, 0, yout);
+      flag = CV_ROOT_RETURN;
+      ca_mem->ca_rootret = SUNFALSE;
+      earlyret = SUNTRUE;
+    }
+
+    /* return if necessary */
+    if (earlyret) {
+      *ncheckPtr = ca_mem->ca_nckpnts;
+      ca_mem->ca_IMnewData = SUNTRUE;
+      ca_mem->ca_ckpntData = ca_mem->ck_mem;
+      ca_mem->ca_np = cv_mem->cv_nst % ca_mem->ca_nsteps + 1;
+      return(flag);
+    }
 
   }
 
@@ -487,7 +508,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
 
     if ( cv_mem->cv_nst % ca_mem->ca_nsteps == 0 ) {
 
-      ca_mem->ck_mem->ck_t1 = *tret;
+      ca_mem->ck_mem->ck_t1 = cv_mem->cv_tn;
 
       /* Create a new check point, load it, and append it to the list */
       tmp = CVAckpntNew(cv_mem);
@@ -500,7 +521,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
       ca_mem->ck_mem = tmp;
       ca_mem->ca_nckpnts++;
       cv_mem->cv_forceSetup = SUNTRUE;
-      
+
       /* Reset i=0 and load dt_mem[0] */
       dt_mem[0]->t = ca_mem->ck_mem->ck_t0;
       ca_mem->ca_IMstore(cv_mem, dt_mem[0]);
@@ -508,7 +529,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
     } else {
 
       /* Load next point in dt_mem */
-      dt_mem[cv_mem->cv_nst % ca_mem->ca_nsteps]->t = *tret;
+      dt_mem[cv_mem->cv_nst % ca_mem->ca_nsteps]->t = cv_mem->cv_tn;
       ca_mem->ca_IMstore(cv_mem, dt_mem[cv_mem->cv_nst % ca_mem->ca_nsteps]);
 
     }
@@ -516,32 +537,42 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
     /* Set t1 field of the current ckeck point structure
        for the case in which there will be no future
        check points */
-    ca_mem->ck_mem->ck_t1 = *tret;
+    ca_mem->ck_mem->ck_t1 = cv_mem->cv_tn;
 
-    /* tfinal is now set to *tret */
-    ca_mem->ca_tfinal = *tret;
+    /* tfinal is now set to tn */
+    ca_mem->ca_tfinal = cv_mem->cv_tn;
 
     /* Return if in CV_ONE_STEP mode */
-    if (iret) break;
+    if (itask == CV_ONE_STEP) break;
+
+    /* CV_NORMAL_STEP returns */
 
-    /* Return if root reached */
-    if ( flag == CV_ROOT_RETURN ) {
-        CVodeGetDky(cv_mem, *tret, 0, yout);
-        break;
-    }
     /* Return if tout reached */
     if ( (*tret - tout)*cv_mem->cv_h >= ZERO ) {
+
+      /* If this was a root return, save the root time to return later */
+      if (flag == CV_ROOT_RETURN) {
+        ca_mem->ca_rootret = SUNTRUE;
+        ca_mem->ca_troot   = *tret;
+      }
+
+      /* Get solution value at tout to return now */
       *tret = tout;
-      CVodeGetDky(cv_mem, tout, 0, yout);
-      /* Reset tretlast in cv_mem so that CVodeGetQuad and CVodeGetSens 
+      flag = CVodeGetDky(cv_mem, tout, 0, yout);
+
+      /* Reset tretlast in cv_mem so that CVodeGetQuad and CVodeGetSens
        * evaluate quadratures and/or sensitivities at the proper time */
       cv_mem->cv_tretlast = tout;
+
       break;
     }
 
-  } /* end of for(;;)() */
+    /* Return if tstop or a root was found */
+    if ( (flag == CV_TSTOP_RETURN) || (flag == CV_ROOT_RETURN) ) break;
 
-  /* Get ncheck from ca_mem */ 
+  } /* end of for(;;) */
+
+  /* Get ncheck from ca_mem */
   *ncheckPtr = ca_mem->ca_nckpnts;
 
   /* Data is available for the last interval */
@@ -554,7 +585,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
 
 
 
-/* 
+/*
  * =================================================================
  * FUNCTIONS FOR BACKWARD PROBLEMS
  * =================================================================
@@ -634,9 +665,9 @@ int CVodeCreateB(void *cvode_mem, int lmmB, int *which)
 
   new_cvB_mem->cv_next = ca_mem->cvB_mem;
   ca_mem->cvB_mem = new_cvB_mem;
-  
+
   /* Return the index of the newly created CVodeBMem object.
-   * This must be passed to CVodeInitB and to other ***B 
+   * This must be passed to CVodeInitB and to other ***B
    * functions to set optional inputs for this backward problem */
 
   *which = ca_mem->ca_nbckpbs;
@@ -646,7 +677,7 @@ int CVodeCreateB(void *cvode_mem, int lmmB, int *which)
   return(CV_SUCCESS);
 }
 
-int CVodeInitB(void *cvode_mem, int which, 
+int CVodeInitB(void *cvode_mem, int which,
                CVRhsFnB fB,
                realtype tB0, N_Vector yB0)
 {
@@ -669,7 +700,7 @@ int CVodeInitB(void *cvode_mem, int which,
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeInitB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check the value of which */
@@ -688,7 +719,7 @@ int CVodeInitB(void *cvode_mem, int which,
   }
 
   cvodeB_mem = (void *) (cvB_mem->cv_mem);
-  
+
   /* Allocate and set the CVODES object */
 
   flag = CVodeInit(cvodeB_mem, CVArhs, tB0, yB0);
@@ -709,7 +740,7 @@ int CVodeInitB(void *cvode_mem, int which,
   return(CV_SUCCESS);
 }
 
-int CVodeInitBS(void *cvode_mem, int which, 
+int CVodeInitBS(void *cvode_mem, int which,
                 CVRhsFnBS fBs,
                 realtype tB0, N_Vector yB0)
 {
@@ -732,7 +763,7 @@ int CVodeInitBS(void *cvode_mem, int which,
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeInitBS", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check the value of which */
@@ -751,7 +782,7 @@ int CVodeInitBS(void *cvode_mem, int which,
   }
 
   cvodeB_mem = (void *) (cvB_mem->cv_mem);
-  
+
   /* Allocate and set the CVODES object */
 
   flag = CVodeInit(cvodeB_mem, CVArhs, tB0, yB0);
@@ -840,7 +871,7 @@ int CVodeSStolerancesB(void *cvode_mem, int which, realtype reltolB, realtype ab
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSStolerancesB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check the value of which */
@@ -889,7 +920,7 @@ int CVodeSVtolerancesB(void *cvode_mem, int which, realtype reltolB, N_Vector ab
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeSVtolerancesB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check the value of which */
@@ -937,7 +968,7 @@ int CVodeQuadInitB(void *cvode_mem, int which,
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadInitB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
@@ -984,7 +1015,7 @@ int CVodeQuadInitBS(void *cvode_mem, int which,
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadInitBS", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
@@ -1030,7 +1061,7 @@ int CVodeQuadReInitB(void *cvode_mem, int which, N_Vector yQB0)
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadReInitB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check the value of which */
@@ -1073,7 +1104,7 @@ int CVodeQuadSStolerancesB(void *cvode_mem, int which, realtype reltolQB, realty
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadSStolerancesB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
@@ -1115,7 +1146,7 @@ int CVodeQuadSVtolerancesB(void *cvode_mem, int which, realtype reltolQB, N_Vect
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadSStolerancesB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
@@ -1143,7 +1174,7 @@ int CVodeQuadSVtolerancesB(void *cvode_mem, int which, realtype reltolQB, N_Vect
  *
  * This routine performs the backward integration towards tBout
  * of all backward problems that were defined.
- * When necessary, it performs a forward integration between two 
+ * When necessary, it performs a forward integration between two
  * consecutive check points to update interpolation data.
  *
  * On a successful return, CVodeB returns CV_SUCCESS.
@@ -1166,7 +1197,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
   int sign, flag=0;
   realtype tfuzz, tBret, tBn;
   booleantype gotCheckpoint, isActive, reachedTBout;
-  
+
   /* Check if cvode_mem exists */
 
   if (cvode_mem == NULL) {
@@ -1325,7 +1356,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
 
       if ( isActive ) {
 
-        /* Store the address of current backward problem memory 
+        /* Store the address of current backward problem memory
          * in ca_mem to be used in the wrapper functions */
         ca_mem->ca_bckpbCrt = tmp_cvB_mem;
 
@@ -1380,7 +1411,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
 
     ck_mem = ck_mem->ck_next;
 
-  } 
+  }
 
   return(flag);
 }
@@ -1403,7 +1434,7 @@ int CVodeGetB(void *cvode_mem, int which, realtype *tret, N_Vector yB)
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeGetB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
 
   ca_mem = cv_mem->cv_adj_mem;
 
@@ -1418,7 +1449,7 @@ int CVodeGetB(void *cvode_mem, int which, realtype *tret, N_Vector yB)
   while (cvB_mem != NULL) {
     if ( which == cvB_mem->cv_index ) break;
     cvB_mem = cvB_mem->cv_next;
-  } 
+  }
 
   N_VScale(ONE, cvB_mem->cv_y, yB);
   *tret = cvB_mem->cv_tout;
@@ -1451,7 +1482,7 @@ int CVodeGetQuadB(void *cvode_mem, int which, realtype *tret, N_Vector qB)
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeGetQuadB", MSGCV_NO_ADJ);
     return(CV_NO_ADJ);
-  } 
+  }
 
   ca_mem = cv_mem->cv_adj_mem;
 
@@ -1466,7 +1497,7 @@ int CVodeGetQuadB(void *cvode_mem, int which, realtype *tret, N_Vector qB)
   while (cvB_mem != NULL) {
     if ( which == cvB_mem->cv_index ) break;
     cvB_mem = cvB_mem->cv_next;
-  } 
+  }
 
   cvodeB_mem = (void *) (cvB_mem->cv_mem);
 
@@ -1474,7 +1505,7 @@ int CVodeGetQuadB(void *cvode_mem, int which, realtype *tret, N_Vector qB)
    * simply return the current value of qB (i.e. the final conditions) */
 
   flag = CVodeGetNumSteps(cvodeB_mem, &nstB);
-  
+
   if (nstB == 0) {
     N_VScale(ONE, cvB_mem->cv_mem->cv_znQ[0], qB);
     *tret = cvB_mem->cv_tout;
@@ -1486,7 +1517,7 @@ int CVodeGetQuadB(void *cvode_mem, int which, realtype *tret, N_Vector qB)
 }
 
 
-/* 
+/*
  * =================================================================
  * PRIVATE FUNCTIONS FOR CHECK POINTS
  * =================================================================
@@ -1495,7 +1526,7 @@ int CVodeGetQuadB(void *cvode_mem, int which, realtype *tret, N_Vector qB)
 /*
  * CVAckpntInit
  *
- * This routine initializes the check point linked list with 
+ * This routine initializes the check point linked list with
  * information from the initial time.
  */
 
@@ -1514,7 +1545,7 @@ static CkpntMem CVAckpntInit(CVodeMem cv_mem)
     free(ck_mem); ck_mem = NULL;
     return(NULL);
   }
-  
+
   ck_mem->ck_zn[1] = N_VClone(cv_mem->cv_tempv);
   if (ck_mem->ck_zn[1] == NULL) {
     N_VDestroy(ck_mem->ck_zn[0]);
@@ -1531,7 +1562,7 @@ static CkpntMem CVAckpntInit(CVodeMem cv_mem)
   ck_mem->ck_nst   = 0;
   ck_mem->ck_q     = 1;
   ck_mem->ck_h     = 0.0;
-  
+
   /* Do we need to carry quadratures */
   ck_mem->ck_quadr = cv_mem->cv_quadr && cv_mem->cv_errconQ;
 
@@ -1585,7 +1616,7 @@ static CkpntMem CVAckpntInit(CVodeMem cv_mem)
       free(ck_mem); ck_mem = NULL;
       return(NULL);
     }
-    
+
     for (is=0; is<cv_mem->cv_Ns; is++)
       cv_mem->cv_cvals[is] = ONE;
 
@@ -1602,7 +1633,7 @@ static CkpntMem CVAckpntInit(CVodeMem cv_mem)
 /*
  * CVAckpntNew
  *
- * This routine allocates space for a new check point and sets 
+ * This routine allocates space for a new check point and sets
  * its data from current values in cv_mem.
  */
 
@@ -1782,7 +1813,7 @@ static CkpntMem CVAckpntNew(CVodeMem cv_mem)
         cv_mem->cv_Zvecs[j*cv_mem->cv_Ns+is] = ck_mem->ck_znS[j][is];
       }
     }
-    
+
     (void) N_VScaleVectorArray(cv_mem->cv_Ns*(cv_mem->cv_q+1),
                                cv_mem->cv_cvals,
                                cv_mem->cv_Xvecs, cv_mem->cv_Zvecs);
@@ -1795,7 +1826,7 @@ static CkpntMem CVAckpntNew(CVodeMem cv_mem)
                                  cv_mem->cv_znS[qmax], ck_mem->ck_znS[qmax]);
     }
   }
-  
+
   if (ck_mem->ck_quadr_sensi) {
     for (j=0; j<=cv_mem->cv_q; j++) {
       for (is=0; is<cv_mem->cv_Ns; is++) {
@@ -1862,8 +1893,8 @@ static void CVAckpntDelete(CkpntMem *ck_memPtr)
   for (j=0;j<=tmp->ck_q;j++) N_VDestroy(tmp->ck_zn[j]);
   if (tmp->ck_zqm != 0) N_VDestroy(tmp->ck_zn[tmp->ck_zqm]);
 
-  /* free N_Vectors for quadratures in tmp 
-   * Note that at the check point at t_initial, only znQ_[0] 
+  /* free N_Vectors for quadratures in tmp
+   * Note that at the check point at t_initial, only znQ_[0]
    * was allocated */
   if (tmp->ck_quadr) {
 
@@ -1873,42 +1904,42 @@ static void CVAckpntDelete(CkpntMem *ck_memPtr)
     } else {
       N_VDestroy(tmp->ck_znQ[0]);
     }
-    
+
   }
 
   /* free N_Vectors for sensitivities in tmp
-   * Note that at the check point at t_initial, only znS_[0] 
+   * Note that at the check point at t_initial, only znS_[0]
    * was allocated */
   if (tmp->ck_sensi) {
-    
+
     if (tmp->ck_next != NULL) {
       for (j=0;j<=tmp->ck_q;j++) N_VDestroyVectorArray(tmp->ck_znS[j], tmp->ck_Ns);
       if (tmp->ck_zqm != 0) N_VDestroyVectorArray(tmp->ck_znS[tmp->ck_zqm], tmp->ck_Ns);
     } else {
       N_VDestroyVectorArray(tmp->ck_znS[0], tmp->ck_Ns);
     }
-    
+
   }
 
   /* free N_Vectors for quadrature sensitivities in tmp
-   * Note that at the check point at t_initial, only znQS_[0] 
+   * Note that at the check point at t_initial, only znQS_[0]
    * was allocated */
   if (tmp->ck_quadr_sensi) {
-    
+
     if (tmp->ck_next != NULL) {
       for (j=0;j<=tmp->ck_q;j++) N_VDestroyVectorArray(tmp->ck_znQS[j], tmp->ck_Ns);
       if (tmp->ck_zqm != 0) N_VDestroyVectorArray(tmp->ck_znQS[tmp->ck_zqm], tmp->ck_Ns);
     } else {
       N_VDestroyVectorArray(tmp->ck_znQS[0], tmp->ck_Ns);
     }
-    
+
   }
 
   free(tmp); tmp = NULL;
 
 }
 
-/* 
+/*
  * =================================================================
  * PRIVATE FUNCTIONS FOR BACKWARD PROBLEMS
  * =================================================================
@@ -1946,7 +1977,7 @@ static void CVAbckpbDelete(CVodeBMem *cvB_memPtr)
 
 }
 
-/* 
+/*
  * =================================================================
  * PRIVATE FUNCTIONS FOR INTERPOLATION
  * =================================================================
@@ -1995,6 +2026,9 @@ static int CVAdataStore(CVodeMem cv_mem, CkpntMem ck_mem)
   /* Run CVode to set following structures in dt_mem[i] */
   i = 1;
   do {
+    // Modified for AMICI
+    // (dt_mem has dimension ca_mem->ca_nsteps)
+    if (i > ca_mem->ca_nsteps) return(CV_FWD_FAIL);
 
     flag = CVode(cv_mem, ck_mem->ck_t1, ca_mem->ca_ytmp, &t, CV_ONE_STEP);
     if (flag < 0) return(CV_FWD_FAIL);
@@ -2020,14 +2054,14 @@ static int CVAdataStore(CVodeMem cv_mem, CkpntMem ck_mem)
  * the check point ck_mem
  */
 
-static int CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem) 
+static int CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem)
 {
   int flag, j, is, qmax, retval;
 
   if (ck_mem->ck_next == NULL) {
 
     /* In this case, we just call the reinitialization routine,
-     * but make sure we use the same initial stepsize as on 
+     * but make sure we use the same initial stepsize as on
      * the first run. */
 
     CVodeSetInitStep(cv_mem, cv_mem->cv_h0u);
@@ -2051,7 +2085,7 @@ static int CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem)
     }
 
   } else {
-    
+
     qmax = cv_mem->cv_qmax;
 
     /* Copy parameters from check point data structure */
@@ -2070,7 +2104,7 @@ static int CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem)
     cv_mem->cv_etamax    = ck_mem->ck_etamax;
     cv_mem->cv_tn        = ck_mem->ck_t0;
     cv_mem->cv_saved_tq5 = ck_mem->ck_saved_tq5;
-    
+
     /* Copy the arrays from check point data structure */
 
     for (j=0; j<=cv_mem->cv_q; j++)
@@ -2127,7 +2161,7 @@ static int CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem)
           cv_mem->cv_Zvecs[j*cv_mem->cv_Ns+is] = cv_mem->cv_znQS[j][is];
         }
       }
-      
+
       retval = N_VScaleVectorArray(cv_mem->cv_Ns*(cv_mem->cv_q+1),
                                    cv_mem->cv_cvals,
                                    cv_mem->cv_Xvecs, cv_mem->cv_Zvecs);
@@ -2146,7 +2180,7 @@ static int CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem)
     for (j=0; j<=L_MAX; j++)        cv_mem->cv_tau[j] = ck_mem->ck_tau[j];
     for (j=0; j<=NUM_TESTS; j++)    cv_mem->cv_tq[j] = ck_mem->ck_tq[j];
     for (j=0; j<=cv_mem->cv_q; j++) cv_mem->cv_l[j] = ck_mem->ck_l[j];
-    
+
     /* Force a call to setup */
 
     cv_mem->cv_forceSetup = SUNTRUE;
@@ -2156,7 +2190,7 @@ static int CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * -----------------------------------------------------------------
  * Functions for interpolation
  * -----------------------------------------------------------------
@@ -2175,7 +2209,7 @@ static int CVAckpntGet(CVodeMem cv_mem, CkpntMem ck_mem)
  * find indx (t is too far beyond limits).
  */
 
-static int CVAfindIndex(CVodeMem cv_mem, realtype t, 
+static int CVAfindIndex(CVodeMem cv_mem, realtype t,
                         long int *indx, booleantype *newpoint)
 {
   CVadjMem ca_mem;
@@ -2206,7 +2240,7 @@ static int CVAfindIndex(CVodeMem cv_mem, realtype t,
     /* look for a new indx to the left */
 
     *newpoint = SUNTRUE;
-    
+
     *indx = ca_mem->ca_ilast;
     for(;;) {
       if ( *indx == 0 ) break;
@@ -2220,7 +2254,7 @@ static int CVAfindIndex(CVodeMem cv_mem, realtype t,
       ca_mem->ca_ilast = *indx;
 
     if ( *indx == 0 ) {
-      /* t is beyond leftmost limit. Is it too far? */  
+      /* t is beyond leftmost limit. Is it too far? */
       if ( SUNRabs(t - dt_mem[0]->t) > FUZZ_FACTOR * cv_mem->cv_uround ) {
         return(CV_GETY_BADT);
       }
@@ -2278,7 +2312,7 @@ int CVodeGetAdjY(void *cvode_mem, realtype t, N_Vector y)
   return(flag);
 }
 
-/* 
+/*
  * -----------------------------------------------------------------
  * Functions specific to cubic Hermite interpolation
  * -----------------------------------------------------------------
@@ -2288,8 +2322,8 @@ int CVodeGetAdjY(void *cvode_mem, realtype t, N_Vector y)
  * CVAhermiteMalloc
  *
  * This routine allocates memory for storing information at all
- * intermediate points between two consecutive check points. 
- * This data is then used to interpolate the forward solution 
+ * intermediate points between two consecutive check points.
+ * This data is then used to interpolate the forward solution
  * at any other time.
  */
 
@@ -2373,12 +2407,12 @@ static booleantype CVAhermiteMalloc(CVodeMem cv_mem)
         allocOK = SUNFALSE;
         break;
       }
-      
+
     }
-    
+
     dt_mem[i]->content = content;
 
-  } 
+  }
 
   /* If an error occurred, deallocate and return */
 
@@ -2413,7 +2447,7 @@ static booleantype CVAhermiteMalloc(CVodeMem cv_mem)
  */
 
 static void CVAhermiteFree(CVodeMem cv_mem)
-{  
+{
   CVadjMem ca_mem;
   DtpntMem *dt_mem;
   HermiteDataMem content;
@@ -2462,7 +2496,7 @@ static int CVAhermiteStorePnt(CVodeMem cv_mem, DtpntMem d)
   /* Load solution */
 
   N_VScale(ONE, cv_mem->cv_zn[0], content->y);
-  
+
   if (ca_mem->ca_IMstoreSensi) {
     for (is=0; is<cv_mem->cv_Ns; is++)
       cv_mem->cv_cvals[is] = ONE;
@@ -2505,8 +2539,8 @@ static int CVAhermiteStorePnt(CVodeMem cv_mem, DtpntMem d)
 /*
  * CVAhermiteGetY ( -> IMget )
  *
- * This routine uses cubic piece-wise Hermite interpolation for 
- * the forward solution vector. 
+ * This routine uses cubic piece-wise Hermite interpolation for
+ * the forward solution vector.
  * It is typically called by the wrapper routines before calling
  * user provided routines (fB, djacB, bjacB, jtimesB, psolB) but
  * can be directly called by the user through CVodeGetAdjY
@@ -2534,12 +2568,12 @@ static int CVAhermiteGetY(CVodeMem cv_mem, realtype t,
   realtype  cvals[4];
   N_Vector  Xvecs[4];
   N_Vector* XXvecs[4];
- 
+
   ca_mem = cv_mem->cv_adj_mem;
   dt_mem = ca_mem->dt_mem;
- 
+
   /* Local value of Ns */
- 
+
   NS = (ca_mem->ca_IMinterpSensi && (yS != NULL)) ? cv_mem->cv_Ns : 0;
 
   /* Get the index in dt_mem */
@@ -2562,7 +2596,7 @@ static int CVAhermiteGetY(CVodeMem cv_mem, realtype t,
                                    content0->yS, yS);
       if (retval != CV_SUCCESS) return (CV_VECTOROP_ERR);
     }
-    
+
     return(CV_SUCCESS);
   }
 
@@ -2581,7 +2615,7 @@ static int CVAhermiteGetY(CVodeMem cv_mem, realtype t,
   }
 
   if (newpoint) {
-    
+
     /* Recompute Y0 and Y1 */
 
     content1 = (HermiteDataMem) (dt_mem[indx]->content);
@@ -2673,7 +2707,7 @@ static int CVAhermiteGetY(CVodeMem cv_mem, realtype t,
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * -----------------------------------------------------------------
  * Functions specific to Polynomial interpolation
  * -----------------------------------------------------------------
@@ -2683,8 +2717,8 @@ static int CVAhermiteGetY(CVodeMem cv_mem, realtype t,
  * CVApolynomialMalloc
  *
  * This routine allocates memory for storing information at all
- * intermediate points between two consecutive check points. 
- * This data is then used to interpolate the forward solution 
+ * intermediate points between two consecutive check points.
+ * This data is then used to interpolate the forward solution
  * at any other time.
  */
 
@@ -2752,7 +2786,7 @@ static booleantype CVApolynomialMalloc(CVodeMem cv_mem)
 
     dt_mem[i]->content = content;
 
-  } 
+  }
 
   /* If an error occurred, deallocate and return */
 
@@ -2848,7 +2882,7 @@ static int CVApolynomialStorePnt(CVodeMem cv_mem, DtpntMem d)
 /*
  * CVApolynomialGetY ( -> IMget )
  *
- * This routine uses polynomial interpolation for the forward solution vector. 
+ * This routine uses polynomial interpolation for the forward solution vector.
  * It is typically called by the wrapper routines before calling
  * user provided routines (fB, djacB, bjacB, jtimesB, psolB)) but
  * can be directly called by the user through CVodeGetAdjY.
@@ -2868,9 +2902,9 @@ static int CVApolynomialGetY(CVodeMem cv_mem, realtype t,
 
   ca_mem = cv_mem->cv_adj_mem;
   dt_mem = ca_mem->dt_mem;
-  
+
   /* Local value of Ns */
- 
+
   NS = (ca_mem->ca_IMinterpSensi && (yS != NULL)) ? cv_mem->cv_Ns : 0;
 
   /* Get the index in dt_mem */
@@ -2987,7 +3021,7 @@ static int CVApolynomialGetY(CVodeMem cv_mem, realtype t,
 
 }
 
-/* 
+/*
  * =================================================================
  * WRAPPERS FOR ADJOINT SYSTEM
  * =================================================================
@@ -2995,11 +3029,11 @@ static int CVApolynomialGetY(CVodeMem cv_mem, realtype t,
 /*
  * CVArhs
  *
- * This routine interfaces to the CVRhsFnB (or CVRhsFnBS) routine 
+ * This routine interfaces to the CVRhsFnB (or CVRhsFnBS) routine
  * provided by the user.
  */
 
-static int CVArhs(realtype t, N_Vector yB, 
+static int CVArhs(realtype t, N_Vector yB,
                   N_Vector yBdot, void *cvode_mem)
 {
   CVodeMem cv_mem;
@@ -3017,7 +3051,7 @@ static int CVArhs(realtype t, N_Vector yB,
 
   if (ca_mem->ca_IMinterpSensi)
     flag = ca_mem->ca_IMget(cv_mem, t, ca_mem->ca_ytmp, ca_mem->ca_yStmp);
-  else 
+  else
     flag = ca_mem->ca_IMget(cv_mem, t, ca_mem->ca_ytmp, NULL);
 
   if (flag != CV_SUCCESS) {
@@ -3042,7 +3076,7 @@ static int CVArhs(realtype t, N_Vector yB,
  * provided by the user.
  */
 
-static int CVArhsQ(realtype t, N_Vector yB, 
+static int CVArhsQ(realtype t, N_Vector yB,
                    N_Vector qBdot, void *cvode_mem)
 {
   CVodeMem cv_mem;
@@ -3061,7 +3095,7 @@ static int CVArhsQ(realtype t, N_Vector yB,
 
   if (ca_mem->ca_IMinterpSensi)
     /* flag = */ ca_mem->ca_IMget(cv_mem, t, ca_mem->ca_ytmp, ca_mem->ca_yStmp);
-  else 
+  else
     /* flag = */ ca_mem->ca_IMget(cv_mem, t, ca_mem->ca_ytmp, NULL);
 
   /* Call the user's RHS function */
