@@ -210,7 +210,7 @@ class Model : public AbstractModel {
     using AbstractModel::fx0_fixedParameters;
     using AbstractModel::fy;
     using AbstractModel::fz;
-    
+
     /**
      * @brief Initialize model properties.
      * @param x Reference to state variables
@@ -546,6 +546,19 @@ class Model : public AbstractModel {
      */
     virtual std::vector<std::string> getObservableNames() const;
 
+    /**
+     * @brief Report whether the model has expression names set.
+     * @return Boolean indicating whether expression names were set. Also
+     * returns `true` if the number of corresponding variables is just zero.
+     */
+    virtual bool hasExpressionNames() const;
+
+    /**
+     * @brief Get names of the expressions.
+     * @return Expression names
+     */
+    virtual std::vector<std::string> getExpressionNames() const;
+
     /**
      * @brief Report whether the model has parameter IDs set.
      * @return Boolean indicating whether parameter IDs were set. Also returns
@@ -597,9 +610,23 @@ class Model : public AbstractModel {
      * @return Observable IDs
      */
     virtual std::vector<std::string> getObservableIds() const;
-    
+
     /**
-     * @brief Checks whether the defined noise model is gaussian, i.e., the nllh is quadratic
+     * @brief Report whether the model has expression IDs set.
+     * @return Boolean indicating whether expression ids were set. Also returns
+     * `true` if the number of corresponding variables is just zero.
+     */
+    virtual bool hasExpressionIds() const;
+
+    /**
+     * @brief Get IDs of the expression.
+     * @return Expression IDs
+     */
+    virtual std::vector<std::string> getExpressionIds() const;
+
+    /**
+     * @brief Checks whether the defined noise model is gaussian, i.e., the nllh
+     * is quadratic
      * @return boolean flag
      */
     virtual bool hasQuadraticLLH() const;
@@ -1298,13 +1325,13 @@ class Model : public AbstractModel {
 
     /** Flag indicating Matlab- or Python-based model generation */
     bool pythonGenerated;
-    
+
     /**
      * @brief getter for dxdotdp (matlab generated)
      * @return dxdotdp
      */
     const AmiVectorArray &get_dxdotdp() const;
-    
+
     /**
      * @brief getter for dxdotdp (python generated)
      * @return dxdotdp
@@ -1642,7 +1669,7 @@ class Model : public AbstractModel {
      * @param x Array with the states
      */
     void fdwdx(realtype t, const realtype *x);
-    
+
     /**
      * @brief Compute self derivative for recurring terms in xdot.
      * @param t Timepoint
@@ -1736,7 +1763,7 @@ class Model : public AbstractModel {
      * @return State vector with negative values replaced by `0` according to
      * stateIsNonNegative
      */
-    N_Vector computeX_pos(const_N_Vector x);
+    const_N_Vector computeX_pos(const_N_Vector x);
 
     /** All variables necessary for function evaluation */
     ModelState state_;
@@ -1752,13 +1779,13 @@ class Model : public AbstractModel {
 
     /** Sparse dwdx temporary storage (dimension: `ndwdx`) */
     mutable SUNMatrixWrapper dwdx_;
-    
+
     /** Sparse dwdp temporary storage (dimension: `ndwdp`) */
     mutable SUNMatrixWrapper dwdp_;
-    
+
     /** Dense Mass matrix (dimension: `nx_solver` x `nx_solver`) */
     mutable SUNMatrixWrapper M_;
-    
+
     /**
      * Temporary storage of `dxdotdp_full` data across functions (Python only)
      * (dimension: `nplist` x `nx_solver`, nnz: dynamic,
@@ -1780,7 +1807,7 @@ class Model : public AbstractModel {
      * type `CSC_MAT`)
      */
     mutable SUNMatrixWrapper dxdotdp_implicit;
-    
+
     /**
      * Temporary storage of `dxdotdx_explicit` data across functions (Python only)
      * (dimension: `nplist` x `nx_solver`, nnz: 'nxdotdotdx_explicit',
@@ -2005,10 +2032,10 @@ class Model : public AbstractModel {
 
     /** Sparse dwdw temporary storage (dimension: `ndwdw`) */
     mutable SUNMatrixWrapper dwdw_;
-    
+
     /** Sparse dwdx implicit temporary storage (dimension: `ndwdx`) */
     mutable std::vector<SUNMatrixWrapper> dwdx_hierarchical_;
-    
+
     /** Recursion */
     int w_recursion_depth_ {0};
 };
