@@ -457,20 +457,20 @@ class ReturnData {
 
     /**
      * @brief initializes storage for residual reporting mode
-     * @param enable_res wheter residuals are to be computed
+     * @param enable_res whether residuals are to be computed
      */
     void initializeResidualReporting(bool enable_res);
 
     /**
      * @brief initializes storage for full reporting mode
-     * @param enable_fim wheter FIM Hessian approximation is to be computed
+     * @param enable_fim whether FIM Hessian approximation is to be computed
      */
     void initializeFullReporting(bool enable_fim);
 
 
     /**
      * @brief initialize values for chi2 and llh and derivatives
-     * @param enable_chi2 wheter chi2 values are to be computed
+     * @param enable_chi2 whether chi2 values are to be computed
      */
     void initializeObjectiveFunction(bool enable_chi2);
 
@@ -642,25 +642,23 @@ class ReturnData {
     /**
      * @brief Extracts output information for events, expects that x_solver and sx_solver were
      * were set appropriately
-     * @param iroot event index
      * @param t event timepoint
      * @param rootidx information about which roots fired (1 indicating fired, 0/-1 for not)
      * @param model model that was used in forward solve
      * @param edata ExpData instance carrying experimental data
      */
-    void getEventOutput(int iroot, realtype t, const std::vector<int> rootidx,
+    void getEventOutput(realtype t, const std::vector<int> rootidx,
                         Model &model, ExpData const *edata);
 
     /**
      * @brief Extracts event information for forward sensitivity analysis, expects that x_solver and
      * sx_solver were set appropriately
-     * @param iroot event index
      * @param ie index of event type
      * @param t event timepoint
      * @param model model that was used in forward solve
      * @param edata ExpData instance carrying experimental data
      */
-    void getEventSensisFSA(int iroot, int ie, realtype t, Model &model,
+    void getEventSensisFSA(int ie, realtype t, Model &model,
                            ExpData const *edata);
 
     /**
@@ -679,7 +677,7 @@ class ReturnData {
      * @param model model that was used for forward/backward simulation
      * @param llhS0 contribution to likelihood for initial state sensitivities
      * @param xB vector with final adjoint state
-     * (exluding conservation laws)
+     * (excluding conservation laws)
      */
     void handleSx0Forward(const Model &model,
                           std::vector<realtype> &llhS0,
