@@ -19,10 +19,17 @@
 #include <boost/iostreams/device/back_inserter.hpp>
 #include <boost/iostreams/stream.hpp>
 
-/* Helper functions and forward declarations for boost::serialization */
+/** @file serialization.h Helper functions and forward declarations for
+ * boost::serialization */
 namespace boost {
 namespace serialization {
 
+/**
+ * @brief Serialize a raw array to a boost archive
+ * @param ar archive
+ * @param p Pointer to array
+ * @param size Size of p
+ */
 template <class Archive, typename T>
 void archiveVector(Archive &ar, T **p, int size) {
     if (Archive::is_loading::value) {
@@ -37,6 +44,12 @@ void archiveVector(Archive &ar, T **p, int size) {
     ar &make_array<T>(*p, size);
 }
 
+#ifndef EXHALE_DOXYGEN_SHOULD_SKIP_THIS
+/**
+ * @brief Serialize amici::Solver to boost archive
+ * @param ar Archive
+ * @param s Solver instance to serialize
+ */
 template <class Archive>
 void serialize(Archive &ar, amici::Solver &s, const unsigned int /*version*/) {
     ar &s.sensi_;
@@ -72,12 +85,21 @@ void serialize(Archive &ar, amici::Solver &s, const unsigned int /*version*/) {
     ar &s.rdata_mode_;
 }
 
-
+/**
+ * @brief Serialize amici::CVodeSolver to boost archive
+ * @param ar Archive
+ * @param s Solver instance to serialize
+ */
 template <class Archive>
 void serialize(Archive &ar, amici::CVodeSolver &s, const unsigned int /*version*/) {
     ar & static_cast<amici::Solver&>(s);
 }
 
+/**
+ * @brief Serialize amici::Model to boost archive
+ * @param ar Archive
+ * @param m Model instance to serialize
+ */
 template <class Archive>
 void serialize(Archive &ar, amici::Model &m, const unsigned int /*version*/) {
     ar &m.nx_rdata;
@@ -91,9 +113,6 @@ void serialize(Archive &ar, amici::Model &m, const unsigned int /*version*/) {
     ar &m.nztrue;
     ar &m.ne;
     ar &m.nw;
-    ar &m.ndwdx;
-    ar &m.ndwdp;
-    ar &m.ndxdotdw;
     ar &m.nnz;
     ar &m.nJ;
     ar &m.ubw;
@@ -115,10 +134,13 @@ void serialize(Archive &ar, amici::Model &m, const unsigned int /*version*/) {
     ar &m.tstart_;
     ar &m.state_is_non_negative_;
     ar &m.pythonGenerated;
-    ar &m.ndxdotdp_explicit;
-    ar &m.ndxdotdp_implicit;
 }
 
+/**
+ * @brief Serialize amici::ReturnData to boost archive
+ * @param ar Archive
+ * @param r ReturnData instance to serialize
+ */
 
 template <class Archive>
 void serialize(Archive &ar, amici::ReturnData &r, const unsigned int /*version*/) {
@@ -190,22 +212,23 @@ void serialize(Archive &ar, amici::ReturnData &r, const unsigned int /*version*/
     ar &r.status;
 }
 
-
+#endif
 } // namespace serialization
 } // namespace boost
 
 namespace amici {
 
+/**
+ * @brief Serialize object to char array
+ *
+ * @param data input object
+ * @param size maximum char length
+ *
+ * @return The object serialized as char
+ */
 template <typename T>
 char *serializeToChar(T const& data, int *size) {
-    /**
-     * @brief Serialize object to char array
-     *
-     * @param data input object
-     * @param size maximum char length
-     *
-     * @return The object serialized as char
-     */
+
     try {
         std::string serialized;
         ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
@@ -228,17 +251,17 @@ char *serializeToChar(T const& data, int *size) {
 }
 
 
+/**
+ * @brief Deserialize object that has been serialized using serializeToChar
+ *
+ * @param buffer serialized object
+ * @param size length of buffer
+ *
+ * @return The deserialized object
+ */
 
 template <typename T>
 T deserializeFromChar(const char *buffer, int size) {
-    /**
-     * @brief Deserialize object that has been serialized using serializeToChar
-     *
-     * @param buffer serialized object
-     * @param size length of buffer
-     *
-     * @return The deserialized object
-     */
     try {
         ::boost::iostreams::basic_array_source<char> device(buffer, size);
         ::boost::iostreams::stream<::boost::iostreams::basic_array_source<char>> s(
@@ -253,16 +276,16 @@ T deserializeFromChar(const char *buffer, int size) {
     }
 }
 
+/**
+ * @brief Serialize object to string
+ *
+ * @param data input object
+ *
+ * @return The object serialized as string
+ */
 
 template <typename T>
 std::string serializeToString(T const& data) {
-    /**
-     * @brief Serialize object to string
-     *
-     * @param data input object
-     *
-     * @return The object serialized as string
-     */
     try {
         std::string serialized;
         ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
@@ -280,15 +303,16 @@ std::string serializeToString(T const& data) {
     }
 }
 
+/**
+ * @brief Serialize object to std::vector<char>
+ *
+ * @param data input object
+ *
+ * @return The object serialized as std::vector<char>
+ */
+
 template <typename T>
 std::vector<char> serializeToStdVec(T const& data) {
-    /**
-     * @brief Serialize object to std::vector<char>
-     *
-     * @param data input object
-     *
-     * @return The object serialized as std::vector<char>
-     */
     try{
         std::string serialized;
         ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
@@ -307,15 +331,16 @@ std::vector<char> serializeToStdVec(T const& data) {
     }
 }
 
+/**
+ * @brief Deserialize object that has been serialized using serializeToString
+ *
+ * @param serialized serialized object
+ *
+ * @return The deserialized object
+ */
+
 template <typename T>
 T deserializeFromString(std::string const& serialized) {
-    /**
-     * @brief Deserialize object that has been serialized using serializeToString
-     *
-     * @param serialized serialized object
-     *
-     * @return The deserialized object
-     */
     try{
         ::boost::iostreams::basic_array_source<char> device(serialized.data(),
                                                           serialized.size());
