@@ -14,7 +14,11 @@ namespace amici {
 
 /** Since const N_Vector is not what we want */
 using const_N_Vector =
-    std::add_const<typename std::remove_pointer<N_Vector>::type *>::type;
+    std::add_const<typename std::remove_pointer<N_Vector>::type>::type *;
+
+inline const realtype* N_VGetArrayPointerConst(const_N_Vector x) {
+    return N_VGetArrayPointer(const_cast<N_Vector>(x));
+}
 
 /** AmiVector class provides a generic interface to the NVector_Serial struct */
 class AmiVector {
@@ -260,7 +264,7 @@ class AmiVectorArray {
      * @param pos index of corresponding AmiVector
      * @return N_Vector
      */
-    const N_Vector getNVector(int pos) const;
+    const_N_Vector getNVector(int pos) const;
 
     /**
      * @brief accessor to AmiVector elements
