@@ -20,7 +20,7 @@ AbstractModel::fx0(realtype* /*x0*/,
                    const realtype* /*p*/,
                    const realtype* /*k*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -59,7 +59,7 @@ AbstractModel::fsx0(realtype* /*sx0*/,
                     const realtype* /*k*/,
                     const int /*ip*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -81,7 +81,7 @@ AbstractModel::fstau(realtype* /*stau*/,
                      const int /*ip*/,
                      const int /*ie*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -95,7 +95,7 @@ AbstractModel::fy(realtype* /*y*/,
                   const realtype* /*h*/,
                   const realtype* /*w*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -111,7 +111,7 @@ AbstractModel::fdydp(realtype* /*dydp*/,
                      const realtype* /*w*/,
                      const realtype* /*dwdp*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -126,7 +126,7 @@ AbstractModel::fdydx(realtype* /*dydx*/,
                      const realtype* /*w*/,
                      const realtype* /*dwdx*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -140,7 +140,7 @@ AbstractModel::fz(realtype* /*z*/,
                   const realtype* /*k*/,
                   const realtype* /*h*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -156,7 +156,7 @@ AbstractModel::fsz(realtype* /*sz*/,
                    const realtype* /*sx*/,
                    const int /*ip*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -170,7 +170,7 @@ AbstractModel::frz(realtype* /*rz*/,
                    const realtype* /*k*/,
                    const realtype* /*h*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -186,7 +186,7 @@ AbstractModel::fsrz(realtype* /*srz*/,
                     const realtype* /*sx*/,
                     const int /*ip*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -201,7 +201,7 @@ AbstractModel::fdzdp(realtype* /*dzdp*/,
                      const realtype* /*h*/,
                      const int /*ip*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -215,7 +215,7 @@ AbstractModel::fdzdx(realtype* /*dzdx*/,
                      const realtype* /*k*/,
                      const realtype* /*h*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -230,7 +230,7 @@ AbstractModel::fdrzdp(realtype* /*drzdp*/,
                       const realtype* /*h*/,
                       const int /*ip*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -244,7 +244,7 @@ AbstractModel::fdrzdx(realtype* /*drzdx*/,
                       const realtype* /*k*/,
                       const realtype* /*h*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -260,7 +260,7 @@ AbstractModel::fdeltax(realtype* /*deltax*/,
                        const realtype* /*xdot*/,
                        const realtype* /*xdot_old*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -280,7 +280,7 @@ AbstractModel::fdeltasx(realtype* /*deltasx*/,
                         const realtype* /*sx*/,
                         const realtype* /*stau*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -297,7 +297,7 @@ AbstractModel::fdeltaxB(realtype* /*deltaxB*/,
                         const realtype* /*xdot_old*/,
                         const realtype* /*xB*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -315,7 +315,7 @@ AbstractModel::fdeltaqB(realtype* /*deltaqB*/,
                         const realtype* /*xdot_old*/,
                         const realtype* /*xB*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -326,7 +326,7 @@ AbstractModel::fsigmay(realtype* /*sigmay*/,
                        const realtype* /*p*/,
                        const realtype* /*k*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -338,7 +338,7 @@ AbstractModel::fdsigmaydp(realtype* /*dsigmaydp*/,
                           const realtype* /*k*/,
                           const int /*ip*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -349,7 +349,7 @@ AbstractModel::fsigmaz(realtype* /*sigmaz*/,
                        const realtype* /*p*/,
                        const realtype* /*k*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -361,7 +361,7 @@ AbstractModel::fdsigmazdp(realtype* /*dsigmazdp*/,
                           const realtype* /*k*/,
                           const int /*ip*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -375,7 +375,7 @@ AbstractModel::fJy(realtype* /*nllh*/,
                    const realtype* /*sigmay*/,
                    const realtype* /*my*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -389,7 +389,7 @@ AbstractModel::fJz(realtype* /*nllh*/,
                    const realtype* /*sigmaz*/,
                    const realtype* /*mz*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -402,7 +402,7 @@ AbstractModel::fJrz(realtype* /*nllh*/,
                     const realtype* /*z*/,
                     const realtype* /*sigmaz*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -416,7 +416,23 @@ AbstractModel::fdJydy(realtype* /*dJydy*/,
                       const realtype* /*sigmay*/,
                       const realtype* /*my*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
+}
+
+void
+AbstractModel::fdJydy_colptrs(SUNMatrixWrapper &/*indexptrs*/,
+                              int /*index*/) {
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
+}
+
+void
+AbstractModel::fdJydy_rowvals(SUNMatrixWrapper & /*indexptrs*/,
+                              int /*index*/) {
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -430,7 +446,7 @@ AbstractModel::fdJydsigma(realtype* /*dJydsigma*/,
                           const realtype* /*sigmay*/,
                           const realtype* /*my*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -444,7 +460,7 @@ AbstractModel::fdJzdz(realtype* /*dJzdz*/,
                       const realtype* /*sigmaz*/,
                       const realtype* /*mz*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -458,7 +474,7 @@ AbstractModel::fdJzdsigma(realtype* /*dJzdsigma*/,
                           const realtype* /*sigmaz*/,
                           const realtype* /*mz*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -471,7 +487,7 @@ AbstractModel::fdJrzdz(realtype* /*dJrzdz*/,
                        const realtype* /*rz*/,
                        const realtype* /*sigmaz*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -484,7 +500,7 @@ AbstractModel::fdJrzdsigma(realtype* /*dJrzdsigma*/,
                            const realtype* /*rz*/,
                            const realtype* /*sigmaz*/)
 {
-    throw AmiException("Requested functionality is not supported as (%s) is "
+    throw AmiException("Requested functionality is not supported as %s is "
                        "not implemented for this model!",
                        __func__);
 }
@@ -498,7 +514,9 @@ AbstractModel::fw(realtype* /*w*/,
                   const realtype* /*h*/,
                   const realtype* /*tcl*/)
 {
-    // no-op default implementation
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
 }
 
 void
@@ -512,19 +530,25 @@ AbstractModel::fdwdp(realtype* /*dwdp*/,
                      const realtype* /*tcl*/,
                      const realtype* /*stcl*/)
 {
-    // no-op default implementation
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
 }
 
 void
-AbstractModel::fdwdp_colptrs(sunindextype* /*indexptrs*/)
+AbstractModel::fdwdp_colptrs(SUNMatrixWrapper &/*dwdp*/)
 {
-    // no-op default implementation
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
 }
 
 void
-AbstractModel::fdwdp_rowvals(sunindextype* /*indexvals*/)
+AbstractModel::fdwdp_rowvals(SUNMatrixWrapper &/*dwdp*/)
 {
-    // no-op default implementation
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
 }
 
 void
@@ -539,7 +563,9 @@ AbstractModel::fdwdp(realtype* /*dwdp*/,
                      const realtype* /*stcl*/,
                      const int /*ip*/)
 {
-    // no-op default implementation
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
 }
 
 void
@@ -552,19 +578,51 @@ AbstractModel::fdwdx(realtype* /*dwdx*/,
                      const realtype* /*w*/,
                      const realtype* /*tcl*/)
 {
-    // no-op default implementation
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
 }
 
 void
-AbstractModel::fdwdx_colptrs(sunindextype* /*indexptrs*/)
+AbstractModel::fdwdx_colptrs(SUNMatrixWrapper &/*dwdx*/)
 {
-    // no-op default implementation
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
 }
 
 void
-AbstractModel::fdwdx_rowvals(sunindextype* /*indexvals*/)
+AbstractModel::fdwdx_rowvals(SUNMatrixWrapper &/*dwdx*/)
 {
-    // no-op default implementation
+    throw AmiException("Requested functionality is not supported as %s is "
+                       "not implemented for this model!",
+                       __func__);
+}
+
+void
+AbstractModel::fdwdw(realtype */*dwdw*/,
+                     realtype /*t*/,
+                     const realtype */*x*/,
+                     const realtype */*p*/,
+                     const realtype */*k*/,
+                     const realtype */*h*/,
+                     const realtype */*w*/,
+                     const realtype */*tcl*/) {
+    throw AmiException("Requested functionality is not supported as %s "
+                       "is not implemented for this model!",
+                       __func__);
+}
+
+void AbstractModel::fdwdw_colptrs(SUNMatrixWrapper &/*dwdw*/) {
+    throw AmiException("Requested functionality is not supported as %s "
+                       "is not implemented for this model!",
+                       __func__);
+}
+
+void AbstractModel::fdwdw_rowvals(SUNMatrixWrapper &/*dwdw*/) {
+    throw AmiException("Requested functionality is not supported as %s "
+                       "is not implemented for this model!",
+                       __func__);
 }
 
 } // namespace amici
