@@ -20,7 +20,7 @@ namespace amici {
  * @param ids vector of name/ids of (fixed)Parameters
  * @param values values of the (fixed)Parameters
  * @param id name/id to look for in the vector
- * @param variable_name string indicating what variable we are lookin at
+ * @param variable_name string indicating what variable we are looking at
  * @param id_name string indicating whether name or id was specified
  * @return value of the selected parameter
  */
@@ -42,7 +42,7 @@ static realtype getValueById(std::vector<std::string> const &ids,
  * @param values values of the (fixed)Parameters
  * @param value for the selected parameter
  * @param id name/id to look for in the vector
- * @param variable_name string indicating what variable we are lookin at
+ * @param variable_name string indicating what variable we are looking at
  * @param id_name string indicating whether name or id was specified
  */
 static void setValueById(std::vector<std::string> const &ids,
@@ -63,7 +63,7 @@ static void setValueById(std::vector<std::string> const &ids,
  * @param values values of the (fixed)Parameters
  * @param value for the selected parameter
  * @param regex string according to which names/ids are to be matched
- * @param variable_name string indicating what variable we are lookin at
+ * @param variable_name string indicating what variable we are looking at
  * @param id_name string indicating whether name or id was specified
  * @return number of matched names/ids
  */
@@ -514,6 +514,14 @@ std::vector<std::string> Model::getObservableNames() const {
     return std::vector<std::string>();
 }
 
+bool Model::hasExpressionNames() const {
+    return ny == 0 || !getExpressionNames().empty();
+}
+
+std::vector<std::string> Model::getExpressionNames() const {
+    return std::vector<std::string>();
+}
+
 bool Model::hasParameterIds() const {
     return np() == 0 || !getParameterIds().empty();
 }
@@ -546,6 +554,15 @@ std::vector<std::string> Model::getObservableIds() const {
     return std::vector<std::string>();
 }
 
+bool Model::hasExpressionIds() const {
+    return ny == 0 || !getExpressionIds().empty();
+}
+
+std::vector<std::string> Model::getExpressionIds() const {
+    return std::vector<std::string>();
+}
+
+
 bool Model::hasQuadraticLLH() const {
     return true;
 }
@@ -572,7 +589,7 @@ std::vector<bool> const &Model::getStateIsNonNegative() const {
 
 void Model::setStateIsNonNegative(std::vector<bool> const &nonNegative) {
     if (nx_solver != nx_rdata) {
-        throw AmiException("Nonnegative states are not supported whith"
+        throw AmiException("Non-negative states are not supported with"
                            " conservation laws enabled");
     }
     if (state_is_non_negative_.size() != static_cast<unsigned long>(nx_rdata)) {
@@ -1855,7 +1872,7 @@ void Model::fw(const realtype t, const realtype *x) {
 void Model::fdwdp(const realtype t, const realtype *x) {
     if (!nw)
         return;
-        
+
     fw(t, x);
     dwdp_.zero();
     if (pythonGenerated) {
@@ -1896,7 +1913,7 @@ void Model::fdwdx(const realtype t, const realtype *x) {
         return;
 
     fw(t, x);
-    
+
     dwdx_.zero();
     if (pythonGenerated) {
         if (!dwdx_hierarchical_.at(0).capacity())
@@ -1939,7 +1956,7 @@ void Model::fdwdw(const realtype t, const realtype *x) {
     fdwdw(dwdw_.data(), t, x, state_.unscaledParameters.data(),
           state_.fixedParameters.data(), state_.h.data(), w_.data(),
           state_.total_cl.data());
-    
+
     if (always_check_finite_) {
         app->checkFinite(gsl::make_span(dwdw_.get()), "dwdw");
     }
@@ -1989,7 +2006,7 @@ void Model::fstotal_cl(realtype *stotal_cl, const realtype *sx_rdata,
     ftotal_cl(stotal_cl, sx_rdata);
 }
 
-N_Vector Model::computeX_pos(const_N_Vector x) {
+const_N_Vector Model::computeX_pos(const_N_Vector x) {
     if (any_state_non_negative_) {
         for (int ix = 0; ix < x_pos_tmp_.getLength(); ++ix) {
             x_pos_tmp_.at(ix) =
