@@ -43,7 +43,7 @@ SteadystateProblem::SteadystateProblem(const Solver &solver, const Model &model)
 
 void SteadystateProblem::workSteadyStateProblem(Solver *solver, Model *model,
                                                 int it) {
-    
+
     /* process solver handling for pre- or postequilibration */
     if (it == -1) {
         /* solver was not run before, set up everything */
@@ -56,8 +56,8 @@ void SteadystateProblem::workSteadyStateProblem(Solver *solver, Model *model,
         /* solver was run before, extract current state from solver */
         solver->writeSolution(&t_, x_, dx_, sx_, xQ_);
     }
-    
-    /* create a Newton solver obejct */
+
+    /* create a Newton solver object */
     auto newtonSolver = NewtonSolver::getSolver(&t_, &x_, *solver, model);
 
     /* Compute steady state and get the computation time */
@@ -76,7 +76,7 @@ void SteadystateProblem::workSteadyStateProblem(Solver *solver, Model *model,
             /* No steady state could be inferred. Store simulation state */
             storeSimulationState(model, solver->getSensitivityOrder() >=
                                  SensitivityOrder::first);
-            throw AmiException("Steady state sensitvitiy computation failed due "
+            throw AmiException("Steady state sensitivity computation failed due "
                                "to unsuccessful factorization of RHS Jacobian");
         }
     }
@@ -102,7 +102,7 @@ void SteadystateProblem::workSteadyStateBackwardProblem(Solver *solver,
     computeSteadyStateQuadrature(newtonSolver.get(), solver, model);
     cpu_timeB_ = (double)((clock() - starttime) * 1000) / CLOCKS_PER_SEC;
 
-    /* Finalize by setting addjoint state to zero (its steady state) */
+    /* Finalize by setting adjoint state to zero (its steady state) */
     xB_.zero();
 }
 
@@ -241,7 +241,7 @@ bool SteadystateProblem::initializeBackwardProblem(Solver *solver,
 void SteadystateProblem::computeSteadyStateQuadrature(NewtonSolver *newtonSolver,
                                                       const Solver *solver,
                                                       Model *model) {
-    /* This routine computes the qudratures:
+    /* This routine computes the quadratures:
          xQB = Integral[ xB(x(t), t, p) * dxdot/dp(x(t), t, p) | dt ]
      As we're in steady state, we have x(t) = x_ss (x_steadystate), hence
          xQB = Integral[ xB(x_ss, t, p) | dt ] * dxdot/dp(x_ss, t, p)
@@ -265,7 +265,7 @@ void SteadystateProblem::computeSteadyStateQuadrature(NewtonSolver *newtonSolver
 void SteadystateProblem::getQuadratureByLinSolve(NewtonSolver *newtonSolver,
                                                  Model *model) {
     /* Computes the integral over the adjoint state xB:
-     If the Jacobian has full rank, this has an anlytical solution, since
+     If the Jacobian has full rank, this has an analytical solution, since
      d/dt[ xB(t) ] = JB^T(x(t), p) xB(t) = JB^T(x_ss, p) xB(t)
      This linear ODE system with time-constant matrix has the solution
      xB(t) = exp( t * JB^T(x_ss, p) ) * xB(0)
@@ -304,7 +304,7 @@ void SteadystateProblem::getQuadratureBySimulation(const Solver *solver,
     /* create a new solver object */
     auto simSolver = createSteadystateSimSolver(solver, model, false, true);
 
-    /* perform integration and qudrature */
+    /* perform integration and quadrature */
     try {
         runSteadystateSimulation(simSolver.get(), model, true);
         hasQuadrature_ = true;
@@ -418,11 +418,12 @@ realtype SteadystateProblem::getWrmsNorm(const AmiVector &x,
                                          AmiVector &ewt) const {
     /* Depending on what convergence we want to check (xdot, sxdot, xQBdot)
        we need to pass ewt[QB], as xdot and xQBdot have different sizes */
-    N_VAbs(x.getNVector(), ewt.getNVector());
+    N_VAbs(const_cast<N_Vector>(x.getNVector()), ewt.getNVector());
     N_VScale(rtol, ewt.getNVector(), ewt.getNVector());
     N_VAddConst(ewt.getNVector(), atol, ewt.getNVector());
     N_VInv(ewt.getNVector(), ewt.getNVector());
-    return N_VWrmsNorm(xdot.getNVector(), ewt.getNVector());
+    return N_VWrmsNorm(const_cast<N_Vector>(xdot.getNVector()),
+                       ewt.getNVector());
 }
 
 bool SteadystateProblem::checkConvergence(const Solver *solver,
@@ -480,7 +481,7 @@ void SteadystateProblem::applyNewtonsMethod(Model *model,
     int ix = 0;
     double gamma = 1.0;
     bool compNewStep = true;
-    
+
     if (model->nx_solver == 0)
         return;
 
@@ -500,7 +501,7 @@ void SteadystateProblem::applyNewtonsMethod(Model *model,
     bool converged = wrms_ < RCONST(1.0);
     while (!converged && i_newtonstep < newtonSolver->max_steps) {
 
-        /* If Newton steps are necessary, compute the inital search direction */
+        /* If Newton steps are necessary, compute the initial search direction */
         if (compNewStep) {
             try {
                 delta_ = xdot_;
@@ -589,8 +590,8 @@ void SteadystateProblem::runSteadystateSimulation(const Solver *solver,
         solver->getSensitivityMethod() > SensitivityMethod::none)
         sensitivityFlag = SensitivityMethod::forward;
     /* If flag for forward sensitivity computation by simulation is not set,
-     disable forward sensitivity integration. Sensitivities will be combputed
-     by newonSolver->computeNewtonSensis then */
+     disable forward sensitivity integration. Sensitivities will be computed
+     by newtonSolver->computeNewtonSensis then */
     if (model->getSteadyStateSensitivityMode() == SteadyStateSensitivityMode::newtonOnly) {
         solver->switchForwardSensisOff();
         sensitivityFlag = SensitivityMethod::none;
@@ -695,8 +696,9 @@ void SteadystateProblem::computeQBfromQ(Model *model, const AmiVector &yQ,
                                            plist, true);
     } else {
         for (int ip=0; ip<model->nplist(); ++ip)
-            yQB[ip] = N_VDotProd(yQ.getNVector(),
-                                 model->get_dxdotdp().getNVector(ip));
+            yQB[ip] = N_VDotProd(
+                const_cast<N_Vector>(yQ.getNVector()),
+                const_cast<N_Vector>(model->get_dxdotdp().getNVector(ip)));
     }
 }
 
